@@ -353,10 +353,11 @@ static int init_shader(AVCodecContext *avctx, FFVulkanContext *s,
     };
     RET(ff_vk_shader_add_descriptor_set(s, shd, desc_set, 2, 0, 0));
 
-    if (dpx->endian)
+    if (dpx->endian && bits > 8)
         GLSLC(0, #define BIG_ENDIAN                                           );
     GLSLF(0, #define COMPONENTS (%i)                          ,dpx->components);
     GLSLF(0, #define BITS_PER_COMP (%i)                                  ,bits);
+    GLSLF(0, #define BITS_LOG2 (%i)                             ,av_log2(bits));
     GLSLF(0, #define NB_IMAGES (%i)                                    ,planes);
     if (unpack) {
         if (bits == 10)
@@ -402,9 +403,7 @@ static int vk_decode_dpx_init(AVCodecContext *avctx)
 
     switch (dpx->pix_fmt) {
     case AV_PIX_FMT_GRAY10:
-    case AV_PIX_FMT_GRAY12:
     case AV_PIX_FMT_GBRAP10:
-    case AV_PIX_FMT_GBRAP12:
     case AV_PIX_FMT_UYVY422:
     case AV_PIX_FMT_YUV444P:
     case AV_PIX_FMT_YUVA444P:
@@ -467,8 +466,6 @@ const FFHWAccel ff_dpx_vulkan_hwaccel = {
     .frame_priv_data_size  = sizeof(DPXVulkanDecodePicture),
     .init                  = &vk_decode_dpx_init,
     .update_thread_context = &ff_vk_update_thread_context,
-    .decode_params         = &ff_vk_params_invalidate,
-    .flush                 = &ff_vk_decode_flush,
     .uninit                = &ff_vk_decode_uninit,
     .frame_params          = &ff_vk_frame_params,
     .priv_data_size        = sizeof(FFVulkanDecodeContext),
