@@ -1146,8 +1146,6 @@ const FFHWAccel ff_ffv1_vulkan_hwaccel = {
     .frame_priv_data_size  = sizeof(FFv1VulkanDecodePicture),
     .init                  = &vk_decode_ffv1_init,
     .update_thread_context = &ff_vk_update_thread_context,
-    .decode_params         = &ff_vk_params_invalidate,
-    .flush                 = &ff_vk_decode_flush,
     .uninit                = &ff_vk_decode_uninit,
     .frame_params          = &ff_vk_frame_params,
     .priv_data_size        = sizeof(FFVulkanDecodeContext),
